import logging
from spaceone.core.manager import BaseManager
from spaceone.dashboard.model import DomainDashboardVersion

_LOGGER = logging.getLogger(__name__)


class DomainDashboardVersionManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_dashboard_version_model: DomainDashboardVersion = self.locator.get_model('DomainDashboardVersion')

    def create_version_by_domain_dashboard_vo(self, domain_dashboard_vo, params):
        def _rollback(version_vo):
            _LOGGER.info(f'[create_domain_dashboard_version._rollback] '
                         f'Delete domain_dashboard_version_vo : {version_vo.version} '
                         f'({version_vo.domain_dashboard_id})')
            version_vo.delete()

        new_params = {
            'domain_dashboard_id': domain_dashboard_vo.domain_dashboard_id,
            'version': domain_dashboard_vo.version,
            'layouts': params.get('layouts') if params.get('layouts') else domain_dashboard_vo.layouts,
            'variables': params.get('variables') if params.get(
                'variables') else domain_dashboard_vo.variables,
            'settings': params.get('settings') if params.get('settings') else domain_dashboard_vo.settings,
            'variables_schema': params.get('variables_schema') if params.get(
                'variables_schema') else domain_dashboard_vo.variables_schema,
            'domain_id': domain_dashboard_vo.domain_id
        }

        version_vo: DomainDashboardVersion = self.domain_dashboard_version_model.create(new_params)
        self.transaction.add_rollback(_rollback, version_vo)
        return version_vo

    def delete_version(self, domain_dashboard_id, version, domain_id):
        version_vo: DomainDashboardVersion = self.get_version(domain_dashboard_id, version, domain_id)
        version_vo.delete()

    def delete_versions_by_domain_dashboard_version_vos(self, domain_dashboard_version_vos):
        domain_dashboard_version_vos.delete()

    def get_version(self, domain_dashboard_id, version, domain_id, only=None):
        return self.domain_dashboard_version_model.get(domain_dashboard_id=domain_dashboard_id, version=version,
                                                       domain_id=domain_id, only=only)

    def list_versions(self, query=None):
        if query is None:
            query = {}
        return self.domain_dashboard_version_model.query(**query)

    def filter_versions(self, **conditions):
        return self.domain_dashboard_version_model.filter(**conditions)



import copy
import logging

from spaceone.core.service import *
from spaceone.dashboard.manager import DomainDashboardManager, DomainDashboardVersionManager
from spaceone.dashboard.model import DomainDashboard, DomainDashboardVersion
from spaceone.dashboard.error import *

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class DomainDashboardService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_dashboard_mgr: DomainDashboardManager = self.locator.get_manager('DomainDashboardManager')
        self.version_mgr: DomainDashboardVersionManager = self.locator.get_manager('DomainDashboardVersionManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'viewers', 'domain_id'])
    def create(self, params):
        """Register domain_dashboard

        Args:
            params (dict): {
                'name': 'str',
                'viewers': 'str',
                'layouts': 'list',
                'variables': 'dict',
                'settings': 'dict',
                'variables_schema': 'dict',
                'labels': 'list',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            domain_dashboard_vo (object)
        """

        if params['viewers'] == 'PUBLIC':
            params['user_id'] = None
        else:
            params['user_id'] = self.transaction.get_meta('user_id')

        domain_dashboard_vo = self.domain_dashboard_mgr.create_domain_dashboard(params)

        version_keys = ['layouts', 'variables', 'variables_schema']
        if any(set(version_keys) & set(params.keys())):
            self.version_mgr.create_version_by_domain_dashboard_vo(domain_dashboard_vo, params)

        return domain_dashboard_vo

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_dashboard_id', 'domain_id'])
    def update(self, params):
        """Update domain_dashboard

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'name': 'str',
                'layouts': 'list',
                'variables': 'dict',
                'settings': 'dict',
                'variables_schema': 'list',
                'labels': 'list',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            domain_dashboard_vo (object)
        """

        domain_dashboard_id = params['domain_dashboard_id']
        domain_id = params['domain_id']

        domain_dashboard_vo: DomainDashboard = self.domain_dashboard_mgr.get_domain_dashboard(domain_dashboard_id,
                                                                                              domain_id)

        if 'name' not in params:
            params['name'] = domain_dashboard_vo.name

        if domain_dashboard_vo.viewers == 'PRIVATE' and \
                domain_dashboard_vo.user_id != self.transaction.get_meta('user_id'):
            raise ERROR_PERMISSION_DENIED()

        if 'settings' in params:
            params['settings'] = self._merge_settings(domain_dashboard_vo.settings, params['settings'])

        version_change_keys = ['layouts', 'variables', 'variables_schema']
        if self._has_version_key_in_params(domain_dashboard_vo, params, version_change_keys):
            self.domain_dashboard_mgr.increase_version(domain_dashboard_vo)
            self.version_mgr.create_version_by_domain_dashboard_vo(domain_dashboard_vo, params)

        return self.domain_dashboard_mgr.update_domain_dashboard_by_vo(params, domain_dashboard_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_dashboard_id', 'domain_id'])
    def delete(self, params):
        """Deregister domain_dashboard

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        domain_dashboard_vo: DomainDashboard = self.domain_dashboard_mgr.get_domain_dashboard(
            params['domain_dashboard_id'], params['domain_id'])

        if domain_dashboard_vo.viewers == 'PRIVATE' and \
                domain_dashboard_vo.user_id != self.transaction.get_meta('user_id'):
            raise ERROR_PERMISSION_DENIED()

        if domain_dashboard_version_vos := self.version_mgr.filter_versions(
                domain_dashboard_id=domain_dashboard_vo.domain_dashboard_id):
            self.version_mgr.delete_versions_by_domain_dashboard_version_vos(domain_dashboard_version_vos)

        self.domain_dashboard_mgr.delete_by_domain_dashboard_vo(domain_dashboard_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_dashboard_id', 'domain_id'])
    def get(self, params):
        """ Get domain_dashboard

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'domain_id': 'str',
                'only': 'list
            }

        Returns:
            domain_dashboard_vo (object)
        """
        domain_dashboard_id = params['domain_dashboard_id']
        domain_id = params['domain_id']

        domain_dashboard_vo = self.domain_dashboard_mgr.get_domain_dashboard(domain_dashboard_id, domain_id,
                                                                             params.get('only'))

        if domain_dashboard_vo.viewers == 'PRIVATE' and \
                domain_dashboard_vo.user_id != self.transaction.get_meta('user_id'):
            raise ERROR_PERMISSION_DENIED()

        return domain_dashboard_vo

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_dashboard_id', 'version', 'domain_id'])
    def delete_version(self, params):
        """ delete version of domain dashboard

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'version': 'int',
                'domain_id': 'str',
            }

        Returns:
            None
        """

        domain_dashboard_id = params['domain_dashboard_id']
        version = params['version']
        domain_id = params['domain_id']

        domain_dashboard_vo = self.domain_dashboard_mgr.get_domain_dashboard(domain_dashboard_id, domain_id)

        if domain_dashboard_vo.viewers == 'PRIVATE' and \
                domain_dashboard_vo.user_id != self.transaction.get_meta('user_id'):
            raise ERROR_PERMISSION_DENIED()

        current_version = domain_dashboard_vo.version
        if current_version == version:
            raise ERROR_LATEST_VERSION(version=version)

        self.version_mgr.delete_version(domain_dashboard_id, version, domain_id)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_dashboard_id', 'version', 'domain_id'])
    def revert_version(self, params):
        """ Revert version of domain dashboard

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'version': 'int',
                'domain_id': 'str',
            }

        Returns:
            domain_dashboard_vo (object)
        """

        domain_dashboard_id = params['domain_dashboard_id']
        version = params['version']
        domain_id = params['domain_id']

        domain_dashboard_vo: DomainDashboard = self.domain_dashboard_mgr.get_domain_dashboard(domain_dashboard_id,
                                                                                              domain_id)

        if domain_dashboard_vo.viewers == 'PRIVATE' and \
                domain_dashboard_vo.user_id != self.transaction.get_meta('user_id'):
            raise ERROR_PERMISSION_DENIED()

        version_vo: DomainDashboardVersion = self.version_mgr.get_version(domain_dashboard_id, version, domain_id)

        params['layouts'] = version_vo.layouts
        params['variables'] = version_vo.variables
        params['variables_schema'] = version_vo.variables_schema

        self.domain_dashboard_mgr.increase_version(domain_dashboard_vo)
        self.version_mgr.create_version_by_domain_dashboard_vo(domain_dashboard_vo, params)

        return self.domain_dashboard_mgr.update_domain_dashboard_by_vo(params, domain_dashboard_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_dashboard_id', 'version', 'domain_id'])
    def get_version(self, params):
        """ Get version of domain dashboard

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'version': 'int',
                'domain_id': 'str',
                'only': 'list
            }

        Returns:
            domain_dashboard_version_vo (object)
        """

        domain_dashboard_id = params['domain_dashboard_id']
        version = params['version']
        domain_id = params['domain_id']

        domain_dashboard_vo: DomainDashboard = self.domain_dashboard_mgr.get_domain_dashboard(domain_dashboard_id,
                                                                                              domain_id)

        if domain_dashboard_vo.viewers == 'PRIVATE' and \
                domain_dashboard_vo.user_id != self.transaction.get_meta('user_id'):
            raise ERROR_PERMISSION_DENIED()

        return self.version_mgr.get_version(domain_dashboard_id, version, domain_id, params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_dashboard_id', 'domain_id'])
    @append_query_filter(['domain_dashboard_id', 'version', 'domain_id'])
    @append_keyword_filter(['domain_dashboard_id', 'version'])
    def list_versions(self, params):
        """ List versions of domain dashboard

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'version': 'int',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            domain_dashboard_version_vos (object)
            total_count
        """
        domain_dashboard_id = params['domain_dashboard_id']
        domain_id = params['domain_id']

        query = params.get('query', {})
        domain_dashboard_version_vos, total_count = self.version_mgr.list_versions(query)
        domain_dashboard_vo = self.domain_dashboard_mgr.get_domain_dashboard(domain_dashboard_id, domain_id)

        if domain_dashboard_vo.viewers == 'PRIVATE' and \
                domain_dashboard_vo.user_id != self.transaction.get_meta('user_id'):
            raise ERROR_PERMISSION_DENIED()

        return domain_dashboard_version_vos, total_count, domain_dashboard_vo.version

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['domain_dashboard_id', 'name', 'viewers', 'user_id', 'domain_id'])
    @append_keyword_filter(['domain_dashboard_id', 'name'])
    def list(self, params):
        """ List public_dashboards

        Args:
            params (dict): {
                'domain_dashboard_id': 'str',
                'name': 'str',
                'viewers': 'str',
                'user_id': 'str'
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            domain_dashboard_vos (object)
            total_count
        """

        query = params.get('query', {})

        query['filter'] = query.get('filter', [])
        query['filter'].append({
            'k': 'user_id',
            'v': [self.transaction.get_meta('user_id'), None],
            'o': 'in'
        })

        return self.domain_dashboard_mgr.list_domain_dashboards(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['domain_dashboard_id', 'name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list) : 'list of statistics data'

        """
        query = params.get('query', {})

        query['filter'] = query.get('filter', [])
        query['filter'].append({
            'k': 'user_id',
            'v': [self.transaction.get_meta('user_id'), None],
            'o': 'in'
        })

        return self.domain_dashboard_mgr.stat_domain_dashboards(query)

    @staticmethod
    def _has_version_key_in_params(domain_dashboard_vo, params, version_change_keys):
        layouts = domain_dashboard_vo.layouts
        variables = domain_dashboard_vo.variables
        variables_schema = domain_dashboard_vo.variables_schema

        if any(key for key in params if key in version_change_keys):
            if layouts_from_params := params.get('layouts'):
                if layouts != layouts_from_params:
                    return True
            if options_from_params := params.get('variables'):
                if variables != options_from_params:
                    return True
            if schema_from_params := params.get('variables_schema'):
                if schema_from_params != variables_schema:
                    return True
            return False

    @staticmethod
    def _merge_settings(old_settings, new_settings):
        settings = copy.deepcopy(old_settings)

        if old_settings:
            settings.update(new_settings)
            return settings
        else:
            return new_settings



from spaceone.dashboard.model.domain_dashboard_model import DomainDashboard
from spaceone.dashboard.model.project_dashboard_model import ProjectDashboard
from spaceone.dashboard.model.domain_dashboard_version_model import DomainDashboardVersion
from spaceone.dashboard.model.project_dashboard_version_model import ProjectDashboardVersion
from spaceone.dashboard.model.custom_widget_model import CustomWidget


# Database Settings
DATABASES = {
    'default': {
        'db': 'dashboard',
        'host': 'localhost',
        'port': 27017,
        'username': '',
        'password': ''
    }
}

# Cache Settings
CACHES = {
    'default': {
        # Redis Example
        # 'backend': 'spaceone.core.cache.redis_cache.RedisCache',
        # 'host': '<host>',
        # 'port': 6379,
        # 'db': 0
    }
}

# Handler Configuration
HANDLERS = {
    'authentication': [
        # Default Authentication Handler
        # {
        #     'backend': 'spaceone.core.handler.authentication_handler.AuthenticationGRPCHandler',
        #     'uri': 'grpc://identity:50051/v1/Domain/get_public_key'
        # }
    ],
    'authorization': [
        # Default Authorization Handler
        # {
        #     'backend': 'spaceone.core.handler.authorization_handler.AuthorizationGRPCHandler',
        #     'uri': 'grpc://identity:50051/v1/Authorization/verify'
        # }
    ],
    'mutation': [],
    'event': []
}

# Connector Settings
CONNECTORS = {
    'SpaceConnector': {
        'backend': 'spaceone.core.connector.space_connector.SpaceConnector',
        'endpoints': {
            'identity': 'grpc://identity:50051',
        }
    }
}

# Log Settings
LOG = {
}



from spaceone.api.dashboard.v1 import project_dashboard_pb2, project_dashboard_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class ProjectDashboard(BaseAPI, project_dashboard_pb2_grpc.ProjectDashboardServicer):
    pb2 = project_dashboard_pb2
    pb2_grpc = project_dashboard_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            return self.locator.get_info('ProjectDashboardInfo', project_dashboard_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            return self.locator.get_info('ProjectDashboardInfo', project_dashboard_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            project_dashboard_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            return self.locator.get_info('ProjectDashboardInfo', project_dashboard_service.get(params))

    def delete_version(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            project_dashboard_service.delete_version(params)
            return self.locator.get_info('EmptyInfo')

    def revert_version(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            return self.locator.get_info('ProjectDashboardInfo', project_dashboard_service.revert_version(params))

    def get_version(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            return self.locator.get_info('ProjectDashboardVersionInfo', project_dashboard_service.get_version(params))

    def list_versions(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            project_dashboard_version_vos, total_count, dashboard_version = project_dashboard_service.list_versions(
                params)
            return self.locator.get_info('ProjectDashboardVersionsInfo',
                                         project_dashboard_version_vos,
                                         total_count,
                                         minimal=self.get_minimal(params), latest_version=dashboard_version)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            project_dashboard_vos, total_count = project_dashboard_service.list(params)
            return self.locator.get_info('ProjectDashboardsInfo',
                                         project_dashboard_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectDashboardService', metadata) as project_dashboard_service:
            return self.locator.get_info('StatisticsInfo', project_dashboard_service.stat(params))



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class ProjectDashboardVersion(MongoModel):
    project_dashboard_id = StringField(max_length=40)
    version = IntField()
    layouts = ListField(default=[])
    variables = DictField(default={})
    settings = DictField(default={})
    variables_schema = DictField(default={})
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [],
        'minimal_fields': [
            'project_dashboard_id',
            'version',
            'domain_id',
            'created_at',
        ],
        'ordering': ['-version'],
        'indexes': [
            'project_dashboard_id',
            'version',
            'domain_id',
            'created_at'
        ]
    }



import logging
from spaceone.core.manager import BaseManager
from spaceone.dashboard.model.domain_dashboard_model import DomainDashboard

_LOGGER = logging.getLogger(__name__)


class DomainDashboardManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_dashboard_model: DomainDashboard = self.locator.get_model('DomainDashboard')

    def create_domain_dashboard(self, params):
        def _rollback(domain_dashboard_vo):
            _LOGGER.info(f'[create_domain_dashboard._rollback] '
                         f'Delete domain_dashboard_vo : {domain_dashboard_vo.name} '
                         f'({domain_dashboard_vo.domain_dashboard_id})')
            domain_dashboard_vo.delete()

        domain_dashboard_vo: DomainDashboard = self.domain_dashboard_model.create(params)
        self.transaction.add_rollback(_rollback, domain_dashboard_vo)

        return domain_dashboard_vo

    def update_domain_dashboard(self, params):
        domain_dashboard_vo: DomainDashboard = self.get_domain_dashboard(params['domain_dashboard_id'],
                                                                         params['domain_id'])
        return self.update_domain_dashboard_by_vo(params, domain_dashboard_vo)

    def update_domain_dashboard_by_vo(self, params, domain_dashboard_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_domain_dashboard_by_vo._rollback] Revert Data : '
                         f'{old_data["domain_dashboard_id"]}')
            domain_dashboard_vo.update(old_data)

        self.transaction.add_rollback(_rollback, domain_dashboard_vo.to_dict())
        return domain_dashboard_vo.update(params)

    def delete_domain_dashboard(self, domain_dashboard_id, domain_id):
        domain_dashboard_vo: DomainDashboard = self.get_domain_dashboard(domain_dashboard_id, domain_id)
        domain_dashboard_vo.delete()

    def get_domain_dashboard(self, domain_dashboard_id, domain_id, only=None):
        return self.domain_dashboard_model.get(domain_dashboard_id=domain_dashboard_id, domain_id=domain_id, only=only)

    def list_domain_dashboards(self, query=None):
        if query is None:
            query = {}
        return self.domain_dashboard_model.query(**query)

    def stat_domain_dashboards(self, query):
        return self.domain_dashboard_model.stat(**query)

    def increase_version(self, domain_dashboard_vo):
        def _rollback(vo: DomainDashboard):
            _LOGGER.info(f'[increase_version._rollback] Decrease Version : '
                         f'{vo.domain_dashboard_id}')
            vo.decrement('version')

        domain_dashboard_vo.increment('version')
        self.transaction.add_rollback(_rollback, domain_dashboard_vo)

    @staticmethod
    def delete_by_domain_dashboard_vo(domain_dashboard_vo):
        domain_dashboard_vo.delete()



import functools
from spaceone.api.dashboard.v1 import domain_dashboard_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.dashboard.model import DomainDashboardVersion

__all__ = ['DomainDashboardVersionInfo', 'DomainDashboardVersionsInfo']


def DomainDashboardVersionInfo(domain_dashboard_version_vo: DomainDashboardVersion, minimal=False, latest_version=None):
    info = {
        'domain_dashboard_id': domain_dashboard_version_vo.domain_dashboard_id,
        'version': domain_dashboard_version_vo.version,
        'created_at': utils.datetime_to_iso8601(domain_dashboard_version_vo.created_at),
        'domain_id': domain_dashboard_version_vo.domain_id
    }

    if latest_version:
        if latest_version == domain_dashboard_version_vo.version:
            info.update({
                'latest': True
            })
        else:
            info.update({
                'latest': False
            })

    if not minimal:
        info.update({
            'layouts': change_list_value_type(
                domain_dashboard_version_vo.layouts) if domain_dashboard_version_vo.layouts else None,
            'variables': change_struct_type(domain_dashboard_version_vo.variables),
            'settings': change_struct_type(domain_dashboard_version_vo.settings),
            'variables_schema': change_struct_type(domain_dashboard_version_vo.variables_schema)
        })

    return domain_dashboard_pb2.DomainDashboardVersionInfo(**info)


def DomainDashboardVersionsInfo(domain_dashboard_version_vos, total_count, **kwargs):
    return domain_dashboard_pb2.DomainDashboardVersionsInfo(results=list(
        map(functools.partial(DomainDashboardVersionInfo, **kwargs), domain_dashboard_version_vos)),
        total_count=total_count)



from spaceone.dashboard.manager.domain_dashboard_manager import DomainDashboardManager
from spaceone.dashboard.manager.project_dashboard_manager import ProjectDashboardManager
from spaceone.dashboard.manager.domain_dashboard_version_manager import DomainDashboardVersionManager
from spaceone.dashboard.manager.project_dashboard_version_manager import ProjectDashboardVersionManager
from spaceone.dashboard.manager.custom_widget_manager import CustomWidgetManager


import logging
from spaceone.core.manager import BaseManager
from spaceone.dashboard.model.custom_widget_model import CustomWidget

_LOGGER = logging.getLogger(__name__)


class CustomWidgetManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.custom_widget_model: CustomWidget = self.locator.get_model('CustomWidget')

    def create_custom_widget(self, params):
        def _rollback(custom_widget_vo):
            _LOGGER.info(f'[create_custom_widget._rollback] '
                         f'Delete custom_widget_vo : {custom_widget_vo.title} '
                         f'({custom_widget_vo.custom_widget_id})')
            custom_widget_vo.delete()

        custom_widget_vo: CustomWidget = self.custom_widget_model.create(params)
        self.transaction.add_rollback(_rollback, custom_widget_vo)

        return custom_widget_vo

    def update_custom_widget(self, params):
        custom_widget_vo: CustomWidget = self.get_custom_widget(params['custom_widget_id'], params['domain_id'])
        return self.update_custom_widget_by_vo(params, custom_widget_vo)

    def update_custom_widget_by_vo(self, params, custom_widget_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_custom_widget_by_vo._rollback] Revert Data : '
                         f'{old_data["custom_widget_id"]}')
            custom_widget_vo.update(old_data)

        self.transaction.add_rollback(_rollback, custom_widget_vo.to_dict())
        return custom_widget_vo.update(params)

    def delete_custom_widget(self, custom_widget_id, domain_id):
        custom_widget_vo: CustomWidget = self.get_custom_widget(custom_widget_id, domain_id)
        custom_widget_vo.delete()

    def get_custom_widget(self, custom_widget_id, domain_id, only=None):
        return self.custom_widget_model.get(custom_widget_id=custom_widget_id, domain_id=domain_id, only=only)

    def list_custom_widgets(self, query=None):
        if query is None:
            query = {}
        return self.custom_widget_model.query(**query)

    def stat_custom_widgets(self, query):
        return self.custom_widget_model.stat(**query)



import functools
from spaceone.api.dashboard.v1 import custom_widget_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.dashboard.model.custom_widget_model import CustomWidget

__all__ = ['CustomWidgetInfo', 'CustomWidgetsInfo']


def CustomWidgetInfo(custom_widget_vo: CustomWidget, minimal=False):
    info = {
        'custom_widget_id': custom_widget_vo.custom_widget_id,
        'widget_name': custom_widget_vo.widget_name,
        'title': custom_widget_vo.title,
        'version': custom_widget_vo.version,
        'user_id': custom_widget_vo.user_id,
        'domain_id': custom_widget_vo.domain_id
    }

    if not minimal:
        info.update({
            'options': change_struct_type(custom_widget_vo.options),
            'settings': change_struct_type(custom_widget_vo.settings),
            'inherit_options': change_struct_type(custom_widget_vo.inherit_options),
            'labels': change_list_value_type(custom_widget_vo.labels),
            'tags': change_struct_type(custom_widget_vo.tags),
            'created_at': utils.datetime_to_iso8601(custom_widget_vo.created_at),
            'updated_at': utils.datetime_to_iso8601(custom_widget_vo.updated_at)
        })

    return custom_widget_pb2.CustomWidgetInfo(**info)


def CustomWidgetsInfo(custom_widget_vos, total_count, **kwargs):
    return custom_widget_pb2.CustomWidgetsInfo(results=list(
        map(functools.partial(CustomWidgetInfo, **kwargs), custom_widget_vos)), total_count=total_count)



import copy
import logging

from spaceone.core.service import *
from spaceone.dashboard.manager import CustomWidgetManager
from spaceone.dashboard.model import CustomWidget

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class CustomWidgetService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.custom_widget_mgr: CustomWidgetManager = self.locator.get_manager('CustomWidgetManager')

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['widget_name', 'title', 'options', 'domain_id'])
    def create(self, params):
        """Register widget

        Args:
            params (dict): {
                'widget_name': 'str',
                'title': 'str',
                'options': 'dict',
                'settings': 'dict',
                'inherit_options': 'dict',
                'labels': 'list',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            custom_widget_vo (object)
        """

        params['user_id'] = self.transaction.get_meta('user_id')

        return self.custom_widget_mgr.create_custom_widget(params)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['custom_widget_id', 'domain_id'])
    def update(self, params):
        """Update widget

        Args:
            params (dict): {
                'custom_widget_id': 'str',
                'title': 'str',
                'options': 'dict',
                'settings': 'dict',
                'inherit_option': 'dict',
                'labels': 'list',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            custom_widget_vo (object)
        """
        custom_widget_id = params['custom_widget_id']
        domain_id = params['domain_id']

        custom_widget_vo: CustomWidget = self.custom_widget_mgr.get_custom_widget(custom_widget_id, domain_id)

        if 'settings' in params:
            params['settings'] = self._merge_settings(custom_widget_vo.settings, params['settings'])

        return self.custom_widget_mgr.update_custom_widget_by_vo(params, custom_widget_vo)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['custom_widget_id', 'domain_id'])
    def delete(self, params):
        """Deregister widget

        Args:
            params (dict): {
                'custom_widget_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            custom_widget_vo (object)
        """
        return self.custom_widget_mgr.delete_custom_widget(params['custom_widget_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['custom_widget_id', 'domain_id'])
    def get(self, params):
        """Get widget

        Args:
            params (dict): {
                'custom_widget_id': 'str',
                'only': 'list',
                'domain_id': 'str'
            }

        Returns:
            custom_widget_vo (object)
        """

        return self.custom_widget_mgr.get_custom_widget(params['custom_widget_id'], params['domain_id'],
                                                        params.get('only'))

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['domain_id'])
    @append_query_filter(['custom_widget_id', 'widget_name', 'title', 'user_id', 'domain_id'])
    @append_keyword_filter(['custom_widget_id', 'title'])
    def list(self, params):
        """List widget

        Args:
            params (dict): {
                'custom_widget_id': 'str',
                'widget_name': 'str',
                'title': 'str',
                'user_id': 'str',
                'query': 'dict',
                'domain_id': 'str'
            }

        Returns:
            custom_widget_vos (object)
        """
        query = params.get('query', {})
        return self.custom_widget_mgr.list_custom_widgets(query)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['custom_widget_id', 'title'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list) : 'list of statistics data'

        """
        query = params.get('query', {})
        return self.custom_widget_mgr.stat_custom_widgets(query)

    @staticmethod
    def _merge_settings(old_settings, new_settings):
        settings = copy.deepcopy(old_settings)

        if old_settings:
            settings.update(new_settings)
            return settings
        else:
            return new_settings



from spaceone.dashboard.service.domain_dashboard_service import DomainDashboardService
from spaceone.dashboard.service.project_dashboard_service import ProjectDashboardService
from spaceone.dashboard.service.custom_widget_service import CustomWidgetService



from spaceone.dashboard.info.common_info import *
from spaceone.dashboard.info.domain_dashboard_info import *
from spaceone.dashboard.info.domain_dashboard_version_info import *
from spaceone.dashboard.info.project_dashboard_info import *
from spaceone.dashboard.info.project_dashboard_version_info import *
from spaceone.dashboard.info.custom_widget_info import *


import functools
from spaceone.api.dashboard.v1 import project_dashboard_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.dashboard.model.project_dashboard_model import ProjectDashboard

__all__ = ['ProjectDashboardInfo', 'ProjectDashboardsInfo']


def ProjectDashboardInfo(project_dashboard_vo: ProjectDashboard, minimal=False):
    info = {
        'project_dashboard_id': project_dashboard_vo.project_dashboard_id,
        'name': project_dashboard_vo.name,
        'viewers': project_dashboard_vo.viewers,
        'version': project_dashboard_vo.version,
        'labels': change_list_value_type(project_dashboard_vo.labels),
        'project_id': project_dashboard_vo.project_id,
        'user_id': project_dashboard_vo.user_id,
        'domain_id': project_dashboard_vo.domain_id
    }

    if not minimal:
        info.update({
            'layouts': change_list_value_type(
                project_dashboard_vo.layouts) if project_dashboard_vo.layouts else None,
            'variables': change_struct_type(project_dashboard_vo.variables),
            'settings': change_struct_type(project_dashboard_vo.settings),
            'variables_schema': change_struct_type(project_dashboard_vo.variables_schema),
            'tags': change_struct_type(project_dashboard_vo.tags),
            'created_at': utils.datetime_to_iso8601(project_dashboard_vo.created_at),
            'updated_at': utils.datetime_to_iso8601(project_dashboard_vo.updated_at)
        })

    return project_dashboard_pb2.ProjectDashboardInfo(**info)


def ProjectDashboardsInfo(project_dashboard_vos, total_count, **kwargs):
    return project_dashboard_pb2.ProjectDashboardsInfo(results=list(
        map(functools.partial(ProjectDashboardInfo, **kwargs), project_dashboard_vos)), total_count=total_count)



import functools
from spaceone.api.dashboard.v1 import domain_dashboard_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.dashboard.model.domain_dashboard_model import DomainDashboard

__all__ = ['DomainDashboardInfo', 'DomainDashboardsInfo']


def DomainDashboardInfo(domain_dashboard_vo: DomainDashboard, minimal=False):
    info = {
        'domain_dashboard_id': domain_dashboard_vo.domain_dashboard_id,
        'name': domain_dashboard_vo.name,
        'viewers': domain_dashboard_vo.viewers,
        'version': domain_dashboard_vo.version,
        'labels': change_list_value_type(domain_dashboard_vo.labels),
        'user_id': domain_dashboard_vo.user_id,
        'domain_id': domain_dashboard_vo.domain_id
    }

    if not minimal:
        info.update({
            'layouts': change_list_value_type(
                domain_dashboard_vo.layouts) if domain_dashboard_vo.layouts else None,
            'variables': change_struct_type(domain_dashboard_vo.variables),
            'settings': change_struct_type(domain_dashboard_vo.settings),
            'variables_schema': change_struct_type(domain_dashboard_vo.variables_schema),
            'tags': change_struct_type(domain_dashboard_vo.tags),
            'created_at': utils.datetime_to_iso8601(domain_dashboard_vo.created_at),
            'updated_at': utils.datetime_to_iso8601(domain_dashboard_vo.updated_at)
        })

    return domain_dashboard_pb2.DomainDashboardInfo(**info)


def DomainDashboardsInfo(domain_dashboard_vos, total_count, **kwargs):
    return domain_dashboard_pb2.DomainDashboardsInfo(results=list(
        map(functools.partial(DomainDashboardInfo, **kwargs), domain_dashboard_vos)), total_count=total_count)



PROTO = {
    'spaceone.dashboard.interface.grpc.v1.domain_dashboard': ['DomainDashboard'],
    'spaceone.dashboard.interface.grpc.v1.project_dashboard': ['ProjectDashboard'],
    'spaceone.dashboard.interface.grpc.v1.custom_widget': ['CustomWidget']
}



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class CustomWidget(MongoModel):
    custom_widget_id = StringField(max_length=40, generate_id='custom-widget', unique=True)
    widget_name = StringField(max_length=40)
    title = StringField(max_length=255)
    version = StringField(max_length=40)
    options = DictField(default={})
    settings = DictField(default={})
    inherit_options = DictField(default={})
    labels = ListField(StringField())
    tags = DictField(default={})
    user_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'widget_name',
            'title',
            'options',
            'settings',
            'inherit_options',
            'labels',
            'tags'
        ],
        'minimal_fields': [
            'custom_widget_id',
            'widget_name',
            'title',
            'version',
            'user_id',
            'domain_id'
        ],
        'ordering': ['title'],
        'indexes': [
            'widget_name',
            'title',
            'version',
            'user_id',
            'domain_id'
        ]
    }



from spaceone.api.dashboard.v1 import domain_dashboard_pb2, domain_dashboard_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class DomainDashboard(BaseAPI, domain_dashboard_pb2_grpc.DomainDashboardServicer):
    pb2 = domain_dashboard_pb2
    pb2_grpc = domain_dashboard_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            return self.locator.get_info('DomainDashboardInfo', domain_dashboard_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            return self.locator.get_info('DomainDashboardInfo', domain_dashboard_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            domain_dashboard_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            return self.locator.get_info('DomainDashboardInfo', domain_dashboard_service.get(params))

    def delete_version(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            domain_dashboard_service.delete_version(params)
            return self.locator.get_info('EmptyInfo')

    def revert_version(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            return self.locator.get_info('DomainDashboardInfo', domain_dashboard_service.revert_version(params))

    def get_version(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            return self.locator.get_info('DomainDashboardVersionInfo', domain_dashboard_service.get_version(params))

    def list_versions(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            domain_dashboard_version_vos, total_count, dashboard_version = domain_dashboard_service.list_versions(params)
            return self.locator.get_info('DomainDashboardVersionsInfo',
                                         domain_dashboard_version_vos,
                                         total_count,
                                         minimal=self.get_minimal(params), latest_version=dashboard_version)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            domain_dashboard_vos, total_count = domain_dashboard_service.list(params)
            return self.locator.get_info('DomainDashboardsInfo',
                                         domain_dashboard_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainDashboardService', metadata) as domain_dashboard_service:
            return self.locator.get_info('StatisticsInfo', domain_dashboard_service.stat(params))



from mongoengine import *
from spaceone.core.error import ERROR_NOT_UNIQUE

from spaceone.core.model.mongo_model import MongoModel


class DomainDashboard(MongoModel):
    domain_dashboard_id = StringField(max_length=40, generate_id='domain-dash', unique=True)
    name = StringField(max_length=100)
    viewers = StringField(max_length=255, choices=('PUBLIC', 'PRIVATE'))
    version = IntField(default=1)
    layouts = ListField(default=[])
    variables = DictField(default={})
    settings = DictField(default={})
    variables_schema = DictField(default={})
    labels = ListField(StringField())
    tags = DictField(default={})
    user_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'name',
            'layouts',
            'variables',
            'settings',
            'variables_schema',
            'labels',
            'tags'
        ],
        'minimal_fields': [
            'domain_dashboard_id',
            'name',
            'viewers',
            'version',
            'user_id',
            'domain_id'
        ],
        'ordering': ['name'],
        'indexes': [
            'name',
            'viewers',
            'labels',
            'user_id',
            'domain_id'
        ]
    }

    @classmethod
    def create(cls, data):
        domain_dashboard_vos = cls.filter(name=data['name'], user_id=data['user_id'], domain_id=data['domain_id'])
        print(domain_dashboard_vos)
        if domain_dashboard_vos.count() > 0:
            raise ERROR_NOT_UNIQUE(key='name', value=data['name'])
        return super().create(data)

    def update(self, data):
        if 'name' in data:
            domain_dashboard_vos = self.filter(name=data['name'], user_id=self.user_id, domain_id=self.domain_id,
                                               domain_dashboard_id__ne=self.domain_dashboard_id)

            if domain_dashboard_vos.count() > 0:
                raise ERROR_NOT_UNIQUE(key='name', value=data['name'])
            else:
                return super().update(data)
        return super().update(data)



from spaceone.api.dashboard.v1 import custom_widget_pb2, custom_widget_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class CustomWidget(BaseAPI, custom_widget_pb2_grpc.CustomWidgetServicer):
    pb2 = custom_widget_pb2
    pb2_grpc = custom_widget_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CustomWidgetService', metadata) as custom_widget_service:
            return self.locator.get_info('CustomWidgetInfo', custom_widget_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CustomWidgetService', metadata) as custom_widget_service:
            return self.locator.get_info('CustomWidgetInfo', custom_widget_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CustomWidgetService', metadata) as custom_widget_service:
            custom_widget_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CustomWidgetService', metadata) as custom_widget_service:
            return self.locator.get_info('CustomWidgetInfo', custom_widget_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CustomWidgetService', metadata) as custom_widget_service:
            custom_widget_vos, total_count = custom_widget_service.list(params)
            return self.locator.get_info('CustomWidgetsInfo',
                                         custom_widget_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CustomWidgetService', metadata) as custom_widget_service:
            return self.locator.get_info('StatisticsInfo', custom_widget_service.stat(params))



import logging
from spaceone.core.manager import BaseManager
from spaceone.dashboard.model.project_dashboard_model import ProjectDashboard

_LOGGER = logging.getLogger(__name__)


class ProjectDashboardManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.project_dashboard_model: ProjectDashboard = self.locator.get_model('ProjectDashboard')

    def create_project_dashboard(self, params):
        def _rollback(project_dashboard_vo):
            _LOGGER.info(f'[create_project_dashboard._rollback] '
                         f'Delete project_dashboard_vo : {project_dashboard_vo.name} '
                         f'({project_dashboard_vo.project_dashboard_id})')
            project_dashboard_vo.delete()

        project_dashboard_vo: ProjectDashboard = self.project_dashboard_model.create(params)
        self.transaction.add_rollback(_rollback, project_dashboard_vo)

        return project_dashboard_vo

    def update_project_dashboard(self, params):
        project_dashboard_vo: ProjectDashboard = self.get_project_dashboard(params['project_dashboard_id'],
                                                                            params['domain_id'])
        return self.update_project_dashboard_by_vo(params, project_dashboard_vo)

    def update_project_dashboard_by_vo(self, params, project_dashboard_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_project_dashboard_by_vo._rollback] Revert Data : '
                         f'{old_data["project_dashboard_id"]}')
            project_dashboard_vo.update(old_data)

        self.transaction.add_rollback(_rollback, project_dashboard_vo.to_dict())
        return project_dashboard_vo.update(params)

    def delete_project_dashboard(self, project_dashboard_id, domain_id):
        project_dashboard_vo: ProjectDashboard = self.get_project_dashboard(project_dashboard_id, domain_id)
        project_dashboard_vo.delete()

    def get_project_dashboard(self, project_dashboard_id, domain_id, only=None):
        return self.project_dashboard_model.get(project_dashboard_id=project_dashboard_id, domain_id=domain_id,
                                                only=only)

    def list_project_dashboards(self, query=None):
        if query is None:
            query = {}
        return self.project_dashboard_model.query(**query)

    def stat_project_dashboards(self, query):
        return self.project_dashboard_model.stat(**query)

    def increase_version(self, project_dashboard_vo):
        def _rollback(vo: ProjectDashboard):
            _LOGGER.info(f'[increase_version._rollback] Decrease Version : '
                         f'{vo.project_dashboard_id}')
            vo.decrement('version')

        project_dashboard_vo.increment('version')
        self.transaction.add_rollback(_rollback, project_dashboard_vo)

    @staticmethod
    def delete_by_project_dashboard_vo(project_dashboard_vo):
        project_dashboard_vo.delete()
